/* Copyright (C) 2014 Czech Technical University in Prague
 *
 * Authors:
 *     - Michal Horn <hornmich@fel.cvut.cz>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * File : sfunction_cansetup.c
 * Abstract:
 *     C-MEX S-function block for RPP CAN bus setup.
 *
 * References:
 *     header.c
 *     trailer.c
 *
 * Compile with:
 *     <matlabroot>/bin/mex sfunction_cansetup.c
 */

/*
%YAML 1.2
---
Name: CAN Setup
Category: CAN
Header: rpp/can.h
Mnemonic: CANC

Inputs:

Outputs:

Parameters:
  - { name: "Baud rate for CAN bus port 1 [bps]", type: "uint32", range: "1 – 10000000" }
  - { name: "Baud rate for CAN bus port 2 [bps]", type: "uint32", range: "1 – 10000000" }
  - { name: "Baud rate for CAN bus port 3 [bps]", type: "uint32", range: "1 – 10000000" }

# Description is in Markdown mark-up
Description: |

  This block configures the CAN bus controllers. Exactly one CAN bus
  configuration block must be in the model if any of the CAN Receive
  or CAN Transmit block is used.

Status:
  Tested:
    - Configuring CAN1, CAN2 and CAN3
  Untested:
  Not working:
    - Receiving at baudrate higher than 700kb
    - External mode - throwing syntax error during compilation

RPP API functions used:
    - rpp_can_init()

Relevant demos:
    - cantransmit
    - can_demo
...
*/


#define S_FUNCTION_NAME sfunction_cansetup
#include "header.c"
#include <stdio.h> 

#define BAUDRATE_MIN	1
#define BAUDRATE_MAX	1000000

#define BAUDRATE_CAN1_PARAM_NAME	"baudrate_can1"
#define BAUDRATE_CAN2_PARAM_NAME	"baudrate_can2"
#define BAUDRATE_CAN3_PARAM_NAME	"baudrate_can3"

/** Identifiers of the block parameters */
enum params{
	PARAM_CAN1_BAUDRATE,
	PARAM_CAN2_BAUDRATE,
	PARAM_CAN3_BAUDRATE,	
	PARAM_COUNT
};

static void mdlInitializeSizes(SimStruct *S)
{
    if (!rppSetNumParams(S, PARAM_COUNT)) {
        return;
    }
 	
	/* No input ports */
    if (!ssSetNumInputPorts(S, 0)) {
        return;
    }
 
	/* No output ports */
    if (!ssSetNumOutputPorts(S, 0)) {
        return;
    }
 
    rppSetStandardOptions(S);
    void CAN_Common_MdlInitSizes(SimStruct *S);
}    


#ifdef MATLAB_MEX_FILE
#define MDL_CHECK_PARAMETERS
static void mdlCheckParameters(SimStruct *S)
{
    /* Check the parameter CAN1 baudrate */
    if (!rppValidParamRange(S, PARAM_CAN1_BAUDRATE, BAUDRATE_MIN, BAUDRATE_MAX)) {
        return;
    }
    /* Check the parameter CAN2 baudrate */
    if (!rppValidParamRange(S, PARAM_CAN2_BAUDRATE, BAUDRATE_MIN, BAUDRATE_MAX)) {
        return;
    }
    /* Check the parameter CAN3 baudrate */
    if (!rppValidParamRange(S, PARAM_CAN3_BAUDRATE, BAUDRATE_MIN, BAUDRATE_MAX)) {
        return;
    }
}
#endif


#ifdef MATLAB_MEX_FILE
#define MDL_SET_WORK_WIDTHS
static void mdlSetWorkWidths(SimStruct *S)
{
    
    if (!ssSetNumRunTimeParams(S, PARAM_COUNT)) {
        return;
    }
    
	ssRegDlgParamAsRunTimeParam(S, PARAM_CAN1_BAUDRATE, PARAM_CAN1_BAUDRATE, BAUDRATE_CAN1_PARAM_NAME, SS_UINT32);     
	ssRegDlgParamAsRunTimeParam(S, PARAM_CAN2_BAUDRATE, PARAM_CAN2_BAUDRATE, BAUDRATE_CAN2_PARAM_NAME, SS_UINT32);
	ssRegDlgParamAsRunTimeParam(S, PARAM_CAN3_BAUDRATE, PARAM_CAN3_BAUDRATE, BAUDRATE_CAN3_PARAM_NAME, SS_UINT32);
}
#endif

#define COMMON_MDLINITIALIZESAMPLETIMES_INHERIT
#define UNUSED_MDLOUTPUTS
#define UNUSED_MDLTERMINATE
#include "trailer.c"
